@@ -1,4 +1,3 @@
-import logging
 import math
 from typing import Dict, List
 
@@ -9,9 +8,6 @@ import torch.nn.functional as F
 import torch.nn.init as init
 
 
-logger = logging.getLogger(__name__)
-
-
 def gaussian_fill_w_gain(tensor, activation, dim_in, min_std=0.0) -> None:
     """ Gaussian initialization with gain."""
     gain = math.sqrt(2) if activation == "relu" else 1
